#include string.hdr
#include fileio.hdr
#include io.hdr
#include system.hdr
#include pick.hdr

#pragma w_func_proc-
#pragma w_get_local-
#pragma w_precision-

vardef
  char    starting_directory, return_directory, run_string
  int     ret
  logical from_current = .f.
  logical all_drives   = .f.
  logical page_pause   = .f.
  logical ask_switch   = .f.
  logical kill_file    = .f.
  int     counter      = 0
enddef

procedure leave
  select_drive( ret )
  chdir( return_directory )
  cancel
endpro

procedure window_string
  parameters const char vstring
  ? vstring
  counter = ( counter + 1 )
  if page_pause
    if counter = 24
      ? "WAITING: Any key to continue..."
      get_key()
      if lastkey() = 27
        leave()
      endif
      counter = 0
    endif
  endif
endpro

function char parse_commandline
  parameters const char cmd_line
  if len( trim( cmd_line ) ) = 0
    ? "Syntax: FFA <filename> <parameters>"
    ? ""
    ? "Parameters:  /C    - search from Current directory"
    ? "             /D    - search all Drives on system"
    ? "             /P    - Pause per screen page"
    ? "             /A    - Ask for switch to found location"
    ? "             /E    - Erase the file when it is found"
    ? ""
    leave()
  else
    if "/C" $ cmd_line
      window_string( "  - From current directory" )
      from_current = .t.
    endif
    if "/D" $ cmd_line
      window_string( "  - Search all drives" )
      all_drives = .t.
    endif
    if "/P" $ cmd_line
      window_string( "  - Pause per page" )
      page_pause = .t.
    endif
    if "/A" $ cmd_line
      window_string( "  - Ask for switch to found location" )
      ask_switch = .t.
    endif
    if "/E" $ cmd_line
      window_string( "  - Erase file when found" )
      sound( 440, 2 )
      window_string( "  * WARNING: Are you sure you want to erase all files found? (Y=Yes)" )
      if upper( chr( get_key() ) ) <> "Y"
        window_string( "  Exiting system!")
        leave()
      endif
      kill_file = .t.
    endif
  endif
  return( substr( cmd_line, 1, at( " ", cmd_line ) - 1 ) )
endpro

procedure find_files
  parameters const char pathname,;
             const char file_skeleton
  vardef
    int  vrow, vcol
    char response
  enddef
  *\ Find and display the files
  if find_first( pathname + file_skeleton, 0x20 )
    repeat
      if kill_file
        window_string( "    " + pathname + find_fstr() + " <-Deleted" )
      else
        window_string( "    " + pathname + find_fstr() )
      endif

      if kill_file
        del( pathname + find_fstr() )
      endif

      if ask_switch
        ?? " : Goto? (Y/N/V)"
        response = upper( chr( get_key() ) )
        if response = "Y"
          select_drive( asc( upper( substr( pathname, 1, 1 ) ) ) - 65 )
          chdir( substr( pathname, 1, len( trim( pathname ) ) - 1 ) )
          cancel
        else
          if response = "V"
            run "TYPE " + pathname + find_fstr() + " |MORE"
            counter = 0
          else
            if asc( response ) = 27
              leave()
            endif
          endif
        endif
      endif

    until .not. find_next()
  endif
endpro

procedure add_dir_list
  parameters const char cur_place,;
             const char file_skeleton
  vardef
    long current_dir
    long cnt
    uint handle
  enddef
  current_dir = pick_init()
  if find_first( cur_place + "*.", 0x10 )
    repeat
      if is_key() = 27
        leave()
      endif
      if trim( find_fstr() ) <> ".\" .and. trim( find_fstr() ) <> "..\"
        if .not. fb_open( handle, cur_place + find_fstr(), &B_READ )
          pick_add( current_dir, find_fstr() )
        else
          fb_close( handle )
        endif
      endif
    until .not. find_next()
    for cnt = 1 to slist_max_( current_dir )
      find_files( cur_place + pick_str( current_dir, cnt ), file_skeleton )
      add_dir_list( cur_place + pick_str( current_dir, cnt ),;
                    file_skeleton )
    next
  endif
  pick_clear( current_dir )
endpro

procedure force_main
  parameters const char command_line
  vardef
    int  cnt
    char file_skeleton
  enddef
  scrn_dos()
  window_string( "File Find Advanced - by Mark Witten" )
  window_string( "" )
  ret = curdrive()
  return_directory = substr( chr( curdrive() + 65 ) + ":\", 1, 2 ) + curdir( 0 )
  file_skeleton = parse_commandline( trim( upper( command_line ) ) )
  window_string( ">>Looking for [" + file_skeleton + "]" )
  *\
  *\  SEARCH DIRECTORIES ON CURRENT DRIVE
  *\
  if .not. from_current
    chdir( chr( curdrive() + 65 ) + ":\" )
  endif
  starting_directory = substr( chr( curdrive() + 65 ) + ":\", 1, 2 ) + curdir( 0 )
  window_string( ">>Searching directories, drive " + chr( curdrive() + 65 ) + ":..." )
  *\ If we are in the root... \*
  if len( trim( starting_directory ) ) = 3
    find_files( chr( curdrive() + 65 ) + ":\", file_skeleton )
    add_dir_list( starting_directory, file_skeleton )
  else
    find_files( starting_directory + "\", file_skeleton )
    add_dir_list( starting_directory + "\", file_skeleton )
  endif
  if all_drives
    *\
    *\  SEARCH ALL HARDDISK DRIVES
    *\
    window_string( ">>Searching other available drives..." )
    for cnt = 2 to 25
      if cnt <> ret
        select_drive( cnt )
        if curdrive() = cnt
          chdir( chr( curdrive() + 65 ) + ":\" )
          window_string( "  >>Searching drive " + chr( curdrive() + 65 ) + ":..." )
          find_files( chr( curdrive() + 65 ) + ":\", file_skeleton )
          add_dir_list( chr( curdrive() + 65 ) + ":\", file_skeleton )
        endif
      endif
    next
  endif
  leave()
endpro
